package com.chinaitop.depot.controller;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.ObjectUtils;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.model.StorageProfitlossSheet;
import com.chinaitop.depot.model.StorageProfitlossSheetExample;
import com.chinaitop.depot.service.StorageProfitlossSheetService;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * @author qingsong.han
 * @description: 损益单管理
 * @create 2018-06-27 11:03
 */
@RestController
@RequestMapping(value = "/storageSheet")
@Api(value = "StorageProfitlossSheetController", description = "损益单管理类")
public class StorageProfitlossSheetController {
    @Resource
    private StorageProfitlossSheetService storageProfitlossSheetService;

    /**
     * 损益单管理列表
     *
     * @param pageNum
     * @param pageSize
     * @return pageInfo
     */
    @RequestMapping(value = "/getList", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "损益单管理列表", notes = "损益单管理列表信息, 支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query", defaultValue = "1"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query", defaultValue = "10"),
            @ApiImplicitParam(name = "houseId", value = "仓房id", paramType = "query"),
            @ApiImplicitParam(name = "warehouseId", value = "货位号id", paramType = "query"),
            @ApiImplicitParam(name = "searchStartDate", value = "条件搜索开始日期(例:2018-07-04)", paramType = "query"),
            @ApiImplicitParam(name = "searchEndDate", value = "条件搜索结束日期(例:2018-07-04)", paramType = "query")
    })
    public PageInfo<StorageProfitlossSheet> getpaymentList(HttpServletRequest request, Integer pageNum, Integer pageSize, Integer houseId, Integer warehouseId, String searchStartDate, String searchEndDate) {
        StorageProfitlossSheetExample example = new StorageProfitlossSheetExample();
        //查询对象条件查询
        StorageProfitlossSheetExample.Criteria createCriteria = example.createCriteria();
        if (null != houseId) {
            createCriteria.andHouseIdEqualTo(houseId);
        }
        if (null != warehouseId) {
            createCriteria.andWarehouseIdEqualTo(warehouseId);
        }
        if (ParameterUtil.isnotnull(searchStartDate) && ParameterUtil.isnotnull(searchEndDate)) {
            createCriteria.andFillTimeBetween(ParameterUtil.string2datetime(searchStartDate+ParameterUtil.startTime), ParameterUtil.string2datetime(searchEndDate+ParameterUtil.endTime));
        }
        String orgId = ObjectUtils.toString(request.getSession().getAttribute("orgId"));
        createCriteria.andOrgIdEqualTo(orgId);
        
        //分页
        PageHelper.startPage(pageNum, pageSize);
        example.setOrderByClause("fill_time desc, id desc");
        List<StorageProfitlossSheet> list = storageProfitlossSheetService.queryByExample(example);
        PageInfo<StorageProfitlossSheet> pageInfo = new PageInfo<StorageProfitlossSheet>(list);
        return pageInfo;
    }

    /**
     * 根据仓号，货位号查询实际库存
     * @param pageNum
     * @param pageSize
     * @param houseId
     * @param warehouseId
     * @return pageInfo
     */
    @RequestMapping(value = "/getLists", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "获取上一次的损益单", notes = "获取上一次的损益单列表信息, 支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query", defaultValue = "1"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query", defaultValue = "10"),
            @ApiImplicitParam(name = "houseId", value = "仓房id", paramType = "query"),
            @ApiImplicitParam(name = "warehouseId", value = "货位号id", paramType = "query")
    })
    public PageInfo<StorageProfitlossSheet> getpaymentLists(Integer pageNum, Integer pageSize, Integer houseId, Integer warehouseId) {
        StorageProfitlossSheetExample example = new StorageProfitlossSheetExample();
        StorageProfitlossSheetExample.Criteria createCriteria = example.createCriteria();
        //查询条件
        if (null != houseId && null != warehouseId) {
            createCriteria.andHouseIdEqualTo(houseId);
            createCriteria.andWarehouseIdEqualTo(warehouseId);
            PageHelper.startPage(pageNum, pageSize);
            example.setOrderByClause("fill_time desc, id desc");
            List<StorageProfitlossSheet> list = storageProfitlossSheetService.queryByExample(example);
            PageInfo<StorageProfitlossSheet> pageInfo = new PageInfo<StorageProfitlossSheet>(list);

            return pageInfo;
        }
        return null;

    }

    /**
     * 保存方法
     *
     * @param storageProfitlossSheetJson
     * @return
     */
    @RequestMapping(value = "/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "损益单管理新增列表", notes = "损益单管理新增信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "storageProfitlossSheetJson", value = "数据对象", paramType = "form"),
            @ApiImplicitParam(name = "storageProfitlossSheetJson", value = "数据对象", paramType = "form")
    })
    public Map<String, Object> save(String storageProfitlossSheetJson, String userInfo) {
        //登录用户名
        JSONObject user = null;
        String realName = "";
        String orgId = "";
        if (null != userInfo && !"".equals(userInfo)) {
        	user = JSONObject.parseObject(userInfo);
        	realName = user.get("realName") == null?"":user.get("realName").toString();
        	orgId = user.get("orgId") == null?"":user.get("orgId").toString();
        }
        Map<String, Object> modelMap = new HashMap<>();
        try {
            // JSON字符串转对象
            ObjectMapper mapper = new ObjectMapper();
            StorageProfitlossSheet storageProfitlossSheets = mapper.readValue(storageProfitlossSheetJson, StorageProfitlossSheet.class);
            storageProfitlossSheets.setOrgId(orgId);
            if (storageProfitlossSheets.getId() == null) {
                storageProfitlossSheetService.add(storageProfitlossSheets, realName); //使用到userInfo.realName
            } else {
                storageProfitlossSheetService.update(storageProfitlossSheets);
            }
            modelMap.put("status", "success");
        } catch (Exception e) {
            modelMap.put("status", "error");
            e.printStackTrace();
        }
        return modelMap;
    }

    /**
     * 根据id查询
     *
     * @param id 主键
     * @return
     */
    @RequestMapping(value = "/edit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "损益单管理查看", notes = "损益单管理查看信息")
    @ApiImplicitParam(name = "id", value = "修改信息id", paramType = "query")
    public StorageProfitlossSheet edit(Integer id) {
        StorageProfitlossSheet storageProfitlossSheet = new StorageProfitlossSheet();
        if (id != null) {
            storageProfitlossSheet = storageProfitlossSheetService.findById(id);
        }
        return storageProfitlossSheet;
    }

    /**
     * 删除(同时删除流程)
     *
     * @param id 主键
     * @return
     * @throws IOException
     */
    @RequestMapping(value = "/remove", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "损益单管理删除", notes = "损益单管理删除列表信息")
    @ApiImplicitParam(name = "integerId", value = "删除的信息Id", paramType = "form")
    public Map<String, Object> remove(Integer id) {
        Map<String, Object> modelMap = new HashMap<>();
        if (id != null) {
            storageProfitlossSheetService.remove(id);
            modelMap.put("status", "success");
        } else {
            modelMap.put("status", "error");
        }
        return modelMap;
    }
}
